@@ -42,6 +42,9 @@
 #undef HAVE_XINERAMA
 #undef HAVE_XFREE_XINERAMA
 #undef HAVE_XINPUT
+#undef UTMP_LINE_ATTACHED
+#undef UTMP_LINE_REMOTE
+#undef UTMP_PSEUDO_DEVICE
 #undef X_SERVER
 #undef X_SERVER_PATH
 #undef X_XNEST_CMD
@@ -51,3 +54,4 @@
 #undef X_PATH
 #undef XSESSION_SHELL
 #undef XEVIE_OPTION
+#undef HAVE_UT_SYSLEN
