@@ -1076,10 +1076,7 @@ gdm_daemon_config_load_xserver (GdmConfig  *config,
 	gboolean        res;
 	GdmConfigValue *value;
 
-	/*
-	 * See if we already loaded a server with this id, skip if
-	 * one already exists.
-	 */
+	/* Do not add xserver if name doesn't exist */
 	if (gdm_daemon_config_find_xserver (name) != NULL) {
 		return;
 	}
@@ -1087,10 +1084,6 @@ gdm_daemon_config_load_xserver (GdmConfig  *config,
 	svr = g_new0 (GdmXserver, 1);
 	svr->id = g_strdup (name);
 
-	if (isdigit (*key)) {
-		svr->number = atoi (key);
-	}
-
 	group = g_strdup_printf ("server-%s", name);
 
 	/* string */
@@ -1103,9 +1096,6 @@ gdm_daemon_config_load_xserver (GdmConfig  *config,
 		svr->command = g_strdup (gdm_config_value_get_string (value));
 	}
 
-	gdm_debug ("Adding new server id=%s name=%s command=%s", name, svr->name, svr->command);
-
-
 	/* bool */
 	res = gdm_config_get_value (config, group, "flexible", &value);
 	if (res) {
@@ -1151,6 +1141,8 @@ gdm_daemon_config_load_xserver (GdmConfig  *config,
 	}
 
 	xservers = g_slist_append (xservers, svr);
+
+	g_free (group);
 }
 
 static void
@@ -1195,66 +1187,78 @@ gdm_daemon_config_ensure_one_xserver (GdmConfig *config)
 static void
 load_xservers_group (GdmConfig *config)
 {
-        char     **keys;
-        gsize      len;
-        int        i;
+	char     **keys;
+	gsize      len;
+	int        i;
 
-        keys = gdm_config_get_keys_for_group (config, GDM_CONFIG_GROUP_SERVERS, &len, NULL);
+	keys = gdm_config_get_keys_for_group (config,
+		GDM_CONFIG_GROUP_SERVERS, &len, NULL);
 
-        /* now construct entries for these groups */
-        for (i = 0; i < len; i++) {
-                GdmConfigEntry  entry;
-                GdmConfigValue *value;
-                char           *new_group;
-                gboolean        res;
-                int             j;
-		const char     *name;
+	/* now construct entries for these groups */
+	for (i = 0; i < len; i++) {
+		GdmConfigEntry   entry;
+		GdmConfigValue  *value;
+		const char      *display_value;
+		const char      *name;
+		char           **value_list;
+		char            *new_group;
+		int              j;
+		gboolean         res;
 
-                entry.group = GDM_CONFIG_GROUP_SERVERS;
-                entry.key = keys[i];
-                entry.type = GDM_CONFIG_VALUE_STRING;
-                entry.default_value = NULL;
-                entry.id = GDM_CONFIG_INVALID_ID;
+	        entry.group         = GDM_CONFIG_GROUP_SERVERS;
+		entry.key           = keys[i];
+		entry.type          = GDM_CONFIG_VALUE_STRING;
+		entry.default_value = NULL;
+		entry.id            = GDM_CONFIG_INVALID_ID;
 
-                gdm_config_add_entry (config, &entry);
-                gdm_config_process_entry (config, &entry, NULL);
+		gdm_config_add_entry (config, &entry);
+		gdm_config_process_entry (config, &entry, NULL);
 
-                res = gdm_config_get_value (config, entry.group, entry.key, &value);
-                if (! res) {
-                        continue;
-                }
-
-		name = gdm_config_value_get_string (value);
-		if (name == NULL || name[0] == '\0') {
-			gdm_config_value_free (value);
+		res = gdm_config_get_value (config, entry.group, entry.key,
+			&value);
+		if (! res) {
 			continue;
 		}
 
-		/* skip servers marked as inactive */
-		if (g_ascii_strcasecmp (name, "inactive") == 0) {
+		display_value = gdm_config_value_get_string (value);
+		value_list = g_strsplit (display_value, " ", -1);
+		if (value_list == NULL || value_list[0] == '\0') {
 			gdm_config_value_free (value);
+			g_strfreev (value_list);
+			continue;
+		}
+		
+		name = value_list[0];
+
+		/* Skip servers marked as inactive */
+		if (name == NULL || name[0] == '\0' ||
+		    g_ascii_strcasecmp (name, "inactive") == 0) {
+			gdm_config_value_free (value);
+			g_strfreev (value_list);
 			continue;
 		}
 
-                new_group = g_strdup_printf ("server-%s", name);
-                for (j = 0; j < G_N_ELEMENTS (gdm_daemon_server_config_entries); j++) {
-                        GdmConfigEntry *srv_entry;
-                        if (gdm_daemon_server_config_entries[j].key == NULL) {
-                                continue;
-                        }
-                        srv_entry = gdm_config_entry_copy (&gdm_daemon_server_config_entries[j]);
-                        g_free (srv_entry->group);
-                        srv_entry->group = g_strdup (new_group);
-                        gdm_config_process_entry (config, srv_entry, NULL);
-                        gdm_config_entry_free (srv_entry);
-                }
+		new_group = g_strdup_printf ("server-%s", name);
+		for (j = 0; j < G_N_ELEMENTS (gdm_daemon_server_config_entries); j++) {
+			GdmConfigEntry *srv_entry;
+			if (gdm_daemon_server_config_entries[j].key == NULL) {
+				continue;
+			}
+			srv_entry = gdm_config_entry_copy (&gdm_daemon_server_config_entries[j]);
+			g_free (srv_entry->group);
+			srv_entry->group = g_strdup (new_group);
+			gdm_config_process_entry (config, srv_entry, NULL);
+			gdm_config_entry_free (srv_entry);
+		}
 		g_free (new_group);
 
-		/* now we can add this server */
-		gdm_daemon_config_load_xserver (config, entry.key, gdm_config_value_get_string (value));
-
+		/* Now we can add this server */
+		gdm_daemon_config_load_xserver (config, entry.key, name);
                 gdm_config_value_free (value);
+		g_strfreev (value_list);
         }
+
+	g_strfreev (keys);
 }
 
 static void
@@ -1315,15 +1319,21 @@ gdm_daemon_config_update_key (const char *keystring)
 
 	/* Shortcut for updating all XDMCP parameters */
 	if (is_key (keystring, "xdmcp/PARAMETERS")) {
-		gdm_daemon_config_update_key (GDM_KEY_DISPLAYS_PER_HOST);
-		gdm_daemon_config_update_key (GDM_KEY_MAX_PENDING);
-		gdm_daemon_config_update_key (GDM_KEY_MAX_WAIT);
-		gdm_daemon_config_update_key (GDM_KEY_MAX_SESSIONS);
-		gdm_daemon_config_update_key (GDM_KEY_INDIRECT);
-		gdm_daemon_config_update_key (GDM_KEY_MAX_INDIRECT);
-		gdm_daemon_config_update_key (GDM_KEY_MAX_WAIT_INDIRECT);
-		gdm_daemon_config_update_key (GDM_KEY_PING_INTERVAL);
-		rc = TRUE;
+		rc = gdm_daemon_config_update_key (GDM_KEY_DISPLAYS_PER_HOST);
+                if (rc == TRUE)
+			rc = gdm_daemon_config_update_key (GDM_KEY_MAX_PENDING);
+                if (rc == TRUE)
+			rc = gdm_daemon_config_update_key (GDM_KEY_MAX_WAIT);
+                if (rc == TRUE)
+			rc = gdm_daemon_config_update_key (GDM_KEY_MAX_SESSIONS);
+                if (rc == TRUE)
+			rc = gdm_daemon_config_update_key (GDM_KEY_INDIRECT);
+                if (rc == TRUE)
+			rc = gdm_daemon_config_update_key (GDM_KEY_MAX_INDIRECT);
+                if (rc == TRUE)
+			rc = gdm_daemon_config_update_key (GDM_KEY_MAX_WAIT_INDIRECT);
+                if (rc == TRUE)
+			rc = gdm_daemon_config_update_key (GDM_KEY_PING_INTERVAL);
 		goto out;
 	}
 
@@ -1344,7 +1354,7 @@ gdm_daemon_config_update_key (const char *keystring)
 		goto out;
 	}
 
-	gdm_config_process_entry (daemon_config, entry, NULL);
+	rc = gdm_config_process_entry (daemon_config, entry, NULL);
 
  out:
 	g_free (group);
@@ -1453,32 +1463,88 @@ have_display_for_number (int number)
 static void
 gdm_daemon_config_load_displays (GdmConfig *config)
 {
-	GSList *l;
+	char       **keys;
+	const char *display_value;
+	gsize      len;
+	int        i,j;
 
-	for (l = xservers; l != NULL; l = l->next) {
-		GdmXserver *xserver;
-		GdmDisplay *disp;
+	keys = gdm_config_get_keys_for_group (config,
+		GDM_CONFIG_GROUP_SERVERS, &len, NULL);
 
-		xserver = l->data;
+        /* now construct entries for these groups */
+        for (i = 0; i < len; i++) {
+		GString         *command     = NULL;
+		GdmDisplay     *disp;
+		GdmConfigValue *value;
+		const char      *name        = NULL;
+		const char      *device_name = NULL;
+		char           **value_list;
+		int             keynum;
+		gboolean        res;
 
-		gdm_debug ("Loading display for key '%d'", xserver->number);
+		name   = keys[i];
 
-		gdm_debug ("Got name for server: %s", xserver->id);
-		/* Do not add if already in the list */
-		if (have_display_for_number (xserver->number)) {
+		if (!isdigit (*name)) {
 			continue;
 		}
 
-		disp = gdm_server_alloc (xserver->number, xserver->id);
+		keynum = atoi (name);
+
+                res = gdm_config_get_value (config, GDM_CONFIG_GROUP_SERVERS,
+			keys[i], &value);
+		if (! res) {
+			continue;
+		}
+
+                display_value = gdm_config_value_get_string (value);
+		value_list    = g_strsplit (display_value, " ", -1);
+
+		if (value_list == NULL || value_list[0] == '\0') {
+			gdm_config_value_free (value);
+			g_strfreev (value_list);
+			continue;
+		}
+
+		command = g_string_new (NULL);
+
+		/*
+		 * Allow an optional device to be passed in as an argument
+		 * with the format "device=/dev/foo".
+		 * In the future, if more per-display configuration is desired, 
+		 * this can be made more sophisticated to handle additional
+		 * arguments.
+		 */
+		j=0;
+		while (value_list[j] != NULL) {
+			if (strncmp (value_list[j], "device=",
+				     strlen ("device=")) == 0) {
+			        device_name = value_list[j] + strlen ("device=");
+			} else {
+				g_string_append (command, value_list[j]);
+				g_string_append (command, " ");
+			}
+			j++;
+                }
+
+		gdm_debug ("Loading display for key '%d'", keynum);
+
+		disp = gdm_display_alloc (keynum, command->str, device_name);
+		g_string_free (command, TRUE);
 		if (disp == NULL) {
+			g_strfreev (value_list);
 			continue;
 		}
 
-		displays = g_slist_insert_sorted (displays, disp, gdm_daemon_config_compare_displays);
-		if (xserver->number > high_display_num) {
-			high_display_num = xserver->number;
+		displays = g_slist_insert_sorted (displays, disp,
+			gdm_daemon_config_compare_displays);
+		if (keynum > high_display_num) {
+			high_display_num = keynum;
 		}
+
+		g_strfreev (value_list);
 	}
+
+	g_ptr_array_free ((GPtrArray*) keys, TRUE);
 }
 
 static gboolean
@@ -1942,7 +2008,7 @@ handle_no_displays (GdmConfig *config,
 		gdm_error (_("%s: XDMCP disabled and no static servers defined. Adding %s on :%d to allow configuration!"),
 			   "gdm_config_parse", server, num);
 
-		d = gdm_server_alloc (num, server);
+		d = gdm_display_alloc (num, server, NULL);
 		d->is_emergency_server = TRUE;
 
 		displays = g_slist_append (displays, d);
@@ -2083,7 +2149,7 @@ gdm_daemon_check_permissions (GdmConfig *config,
 	console_notify = FALSE;
 	gdm_config_get_bool_for_id (daemon_config, GDM_ID_CONSOLE_NOTIFY, &console_notify);
 	auth_path = NULL;
-	gdm_config_get_string_for_id (config, GDM_ID_LOG_DIR, &auth_path);
+	gdm_config_get_string_for_id (config, GDM_ID_SERV_AUTHDIR, &auth_path);
 
 	/* Enter paranoia mode */
 	check_servauthdir (auth_path, &statbuf);
@@ -2170,7 +2236,8 @@ gdm_daemon_config_parse (const char *config_file,
 	}
 	custom_config_file = g_strdup (GDM_CUSTOM_CONF);
 
-	daemon_config = gdm_config_new ();
+	if (daemon_config == NULL)
+		daemon_config = gdm_config_new ();
 
 	gdm_config_set_notify_func (daemon_config, notify_cb, NULL);
 	gdm_config_set_validate_func (daemon_config, validate_cb, NULL);
@@ -2226,9 +2293,6 @@ gdm_daemon_config_parse (const char *config_file,
 
 	GdmUserId = uid;
 	GdmGroupId = gid;
-
-	/* Check that user authentication is properly configured */
-	gdm_verify_check ();
 }
 
 /**
@@ -2267,6 +2331,17 @@ gdm_daemon_config_set_high_display_num (gint val)
 	high_display_num = val;
 }
 
+/**
+ *  gdm_daemon_config_close
+ *
+ *  Cleanup
+ */
+void
+gdm_daemon_config_close (void)
+{
+	gdm_config_free (daemon_config);
+}
+
 /**
  * gdm_is_valid_key
  *
@@ -2328,7 +2403,7 @@ gdm_daemon_config_signal_terminthup_was_notified (void)
  * gdm_daemon_config_get_facefile_from_global
  *
  * These functions are used for accessing the user's face image from their
- * home directory via vicious-extensions.
+ * home directory.
  */
 static gboolean
 check_user_file (const char *path,
@@ -2596,9 +2671,8 @@ is_prog_in_path (const char *prog)
 /**
  * gdm_daemon_config_get_session_exec
  *
- * This function accesses the GDM session desktop file, via vicious
- * extensions and returns the execution command for starting the
- * session.
+ * This function accesses the GDM session desktop file and returns
+ * the execution command for starting the session.
  *
  * Must be called with the PATH set correctly to find session exec.
  */
@@ -2703,11 +2777,81 @@ gdm_daemon_config_get_session_exec (const char *session_name,
 	return ret;
 }
 
+/**
+ * gdm_daemon_config_get_session_xserver_args
+ *
+ * This function accesses the GDM session desktop file and returns
+ * additional Xserver arguments to be used with this session
+ */
+char *
+gdm_daemon_config_get_session_xserver_args (const char *session_name)
+{
+	char        *session_filename;
+	const char  *path_str;
+	char       **search_dirs;
+	GKeyFile    *cfg;
+	static char *xserver_args;
+	static char *cached = NULL;
+	char        *ret;
+
+	cfg = NULL;
+
+	/* clear cache */
+	if (session_name == NULL) {
+		g_free (xserver_args);
+		xserver_args = NULL;
+		g_free (cached);
+		cached = NULL;
+		return NULL;
+	}
+
+	if (cached != NULL && strcmp (ve_sure_string (session_name), ve_sure_string (cached)) == 0)
+		return g_strdup (xserver_args);
+
+	g_free (xserver_args);
+	xserver_args = NULL;
+	g_free (cached);
+	cached = g_strdup (session_name);
+
+	/* Some ugly special casing for legacy "Default.desktop", oh well,
+	 * we changed to "default.desktop" */
+	if (g_ascii_strcasecmp (session_name, "default") == 0 ||
+	    g_ascii_strcasecmp (session_name, "default.desktop") == 0) {
+		session_filename = g_strdup ("default.desktop");
+	} else {
+		session_filename = gdm_ensure_extension (session_name, ".desktop");
+	}
+
+	path_str = gdm_daemon_config_get_value_string (GDM_KEY_SESSION_DESKTOP_DIR);
+	if (path_str == NULL) {
+		gdm_error ("No session desktop directories defined");
+		goto out;
+	}
+
+	search_dirs = g_strsplit (path_str, ":", -1);
+
+	cfg = gdm_common_config_load_from_dirs (session_filename,
+						(const char **)search_dirs,
+						NULL);
+	g_strfreev (search_dirs);
+
+	xserver_args = NULL;
+	gdm_common_config_get_string (cfg, "Desktop Entry/X-Gdm-XserverArgs", &xserver_args, NULL);
+
+ out:
+
+	ret = g_strdup (xserver_args);
+
+	g_key_file_free (cfg);
+
+	return ret;
+}
+
 /**
  * gdm_daemon_config_get_user_session_lang
  *
  * These functions get and set the user's language and setting in their
- * $HOME/.dmrc file via vicious-extensions.
+ * $HOME/.dmrc file.
  */
 void
 gdm_daemon_config_set_user_session_lang (gboolean savesess,
@@ -2720,10 +2864,24 @@ gdm_daemon_config_set_user_session_lang (gboolean savesess,
 	gchar *cfgstr;
 
 	cfgstr = g_build_filename (home_dir, ".dmrc", NULL);
-
 	dmrc = gdm_common_config_load (cfgstr, NULL);
 	if (dmrc == NULL) {
-		return;
+		gint fd = -1;
+		gdm_debug ("file: %s does not exist - creating it", cfgstr);
+		VE_IGNORE_EINTR (fd = g_open (cfgstr,
+			O_CREAT | O_TRUNC | O_RDWR, 0644));
+
+		if (fd < 0)
+			return;
+
+		write (fd, "\n", 2);
+		close (fd);
+		dmrc = gdm_common_config_load (cfgstr, NULL);
+
+		if (dmrc == NULL) {
+			gdm_debug ("failed to open %s after creating it", cfgstr);
+			return;
+		}
 	}
 
 	if (savesess) {
@@ -2770,6 +2928,7 @@ gdm_daemon_config_get_user_session_lang (char      **usrsess,
 
 	save = FALSE;
 	session = NULL;
+	lang = NULL;
 
 	gdm_common_config_get_string (cfg, "Desktop/Session", &session, NULL);
 	if (session == NULL) {
